/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;

/**
 * This class implements the interface {@link IVciEnumDevice} which manages the
 * list of installed VCI device objects. 
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciEnumDevice extends CInterface implements IVciEnumDevice
{
  // Interface Method Index
  private static final int Next         = 3;
  private static final int Skip         = 4;
  private static final int Reset        = 5;
  private static final int AssignEvent  = 6;

  // Interface functions
  private CFunction m_fpNext            = null;
  private CFunction m_fpSkip            = null;
  private CFunction m_fpReset           = null;
  private CFunction m_fpAssignEvent     = null;

  // Device List Event Handle
  private CEvent    m_oDeviceListEvent  = new CEvent();
  private boolean   m_fWaitCanceled     = false;
  private boolean   m_fWaitEvent        = false;

  // Interface Pointer
  private CPointer  m_IVciEnumDevice    = null; //Interface Pointer itself (pThis)

  /**
   * constructor
   * 
   * @param pIVciEnumDevice
   * @throws Throwable
   */
  protected VciEnumDevice(CMalloc pIVciEnumDevice) throws Throwable
  {
    super(pIVciEnumDevice);
    
    try
    {
      int hResult = 0;

      // Retrieve Interface Pointer Address
      m_IVciEnumDevice = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpNext        = new CFunction(m_IVciEnumDevice, Next,        "JNI");
      m_fpSkip        = new CFunction(m_IVciEnumDevice, Skip,        "JNI");
      m_fpReset       = new CFunction(m_IVciEnumDevice, Reset,       "JNI");
      m_fpAssignEvent = new CFunction(m_IVciEnumDevice, AssignEvent, "JNI");

      // Assign Event Handle
      hResult = m_fpAssignEvent.callInt(new Object[]{ m_IVciEnumDevice, m_oDeviceListEvent.GetHandle() });
      if(hResult != 0)
      {
        throw new VciException("IVciEnumDevice->AssignEvent failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_IVciEnumDevice  = null;
      m_fpNext          = null;
      m_fpSkip          = null;
      m_fpReset         = null;
      m_fpAssignEvent   = null;

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_IVciEnumDevice  = null;
      m_fpNext          = null;
      m_fpSkip          = null;
      m_fpReset         = null;
      m_fpAssignEvent   = null;
      
      // Close Device List Event Handle
      if(m_oDeviceListEvent != null)
      {
        m_oDeviceListEvent.CloseHandle();
        m_oDeviceListEvent = null;
      }
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }

  /**
   * @see de.ixxat.vci3.IVciEnumDevice#Next()
   * @throws Throwable
   */
  public VciDeviceInfo Next() throws Throwable
  {
    int hResult = 0;
    
    VciDeviceInfo oVciDeviceInfo = null;

    // Create VCIDEVICEINFO Memory
    CMalloc pVciDeviceInfo = new CMalloc(304);
    try
    {
      hResult = m_fpNext.callInt(new Object[] { m_IVciEnumDevice, new Integer(1), pVciDeviceInfo, null });
      if (hResult != 0)
      {
        throw new VciException("IVciEnumDevice->Next failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        oVciDeviceInfo = new VciDeviceInfo(pVciDeviceInfo);
      }
    } 
    finally
    {
      pVciDeviceInfo.free();
    }
    
    return oVciDeviceInfo;
  }
  
  /**
   * @see de.ixxat.vci3.IVciEnumDevice#Skip(int)
   * @throws Throwable
   */
  public void Skip(int dwNumElements) throws Throwable
  {
    int hResult = 0;
    
    try
    {
      hResult = m_fpSkip.callInt(new Object[] { m_IVciEnumDevice, new Integer(dwNumElements) });
      if (hResult != 0)
      {
        throw new VciException("IVciEnumDevice->Skip failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see de.ixxat.vci3.IVciEnumDevice#Reset()
   * @throws Throwable
   */
  public void Reset() throws Throwable
  {
    int hResult = 0;
    
    try
    {
      hResult = m_fpReset.callInt(new Object[] { m_IVciEnumDevice });
      if (hResult != 0)
      {
        throw new VciException("IVciEnumDevice->Reset failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
    }
  }
  
  /**
   * @see de.ixxat.vci3.IVciEnumDevice#WaitFor(int)
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable
  {
    // reset Wait Canceled Flag
    synchronized (this)
    {
      m_fWaitCanceled = false;
      m_fWaitEvent    = true;
    }//end sync

    // Wait for Event
    m_oDeviceListEvent.WaitFor(dwMilliseconds);

    // check if wait was abandoned by user
    synchronized (this)
    {
      m_fWaitEvent = false;

      if(m_fWaitCanceled)
      {
        m_fWaitCanceled = false;
        throw new VciException("WaitFor abandoned manually", (int)0x80004004 /*E_ABORT*/);
      }
    }//end sync
  }
  
  /**
   * @see de.ixxat.vci3.IVciEnumDevice#CancelWait()
   * @throws Throwable
   */
  public void CancelWait() throws Throwable
  {
    synchronized (this)
    {
      // If a thread is waiting
      if(m_fWaitEvent)
      {
        m_fWaitCanceled = true;

        // Simulate Pulse event
        m_oDeviceListEvent.SetEvent();
        m_oDeviceListEvent.ResetEvent();
      }
    }//end sync
  }
 
}
