/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.vci3.IDisposable;
import de.ixxat.vci3.bal.IBalResource;
import de.ixxat.vci3.bal.lin.LinLineStatus;

/**
 * This interface represents a LIN control unit and is used to control a LIN
 * line. When no longer needed the LIN communication channel object has to be
 * disposed using the {@link IDisposable} interface. Controlling consists of
 * initialization, starting/stopping the LIN line and transmitting LIN IDs.
 * A LIN control object can be obtained via method
 * {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}. The LIN control
 * cannot be opened twice at the same time. Therefore a second try to open the
 * CAN control via {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}
 * fails until the successfully opened CAN control object is discarded.
 * 
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface ILinControl extends IBalResource
{
  /** slave reception mode */
  public static final byte LIN_OPMODE_SLAVE    = 0x00;
  /** master transmission and slave reception mode */
  public static final byte LIN_OPMODE_MASTER   = 0x01;
  /** enable reception of error frames */
  public static final byte LIN_OPMODE_ERRORS   = 0x02;
  
  /** undefined bit-rate */
  public static final short  LIN_BITRATE_UNDEF  = (short)0xFFFF; 
  /** automatic bit-rate detection */
  public static final short  LIN_BITRATE_AUTO   = 0;
  /** lowest specified bit-rate */
  public static final short  LIN_BITRATE_MIN    = 1000;
  /** highest specified bit-rate */
  public static final short  LIN_BITRATE_MAX    = 20000;

  /** 1000 baud */
  public static final short  LIN_BITRATE_1000   = 1000; 
  /** 1200 baud */
  public static final short  LIN_BITRATE_1200   = 1200; 
  /** 2400 baud */
  public static final short  LIN_BITRATE_2400   = 2400; 
  /** 4800 baud */
  public static final short  LIN_BITRATE_4800   = 4800; 
  /** 9600 baud */
  public static final short  LIN_BITRATE_9600   = 9600; 
  /** 10400 baud */
  public static final short  LIN_BITRATE_10400  = 10400;
  /** 19200 baud */
  public static final short  LIN_BITRATE_19200  = 19200;
  /** 20000 baud */
  public static final short  LIN_BITRATE_20000  = 20000;
  

  /**
   * This method initialize the LIN line in the specified operating mode and bit
   * transfer rate. The method also performs a reset of the LIN controller
   * hardware and disables the reception of LIN messages.
   * <p>
   * <strong>Remarks:</strong><blockquote> The operatingMode parameter defines
   * the operating mode of the LIN line. The operating mode can be a one of the
   * following primary operating mode flags: <blockquote>
   * <li> {@link #LIN_OPMODE_SLAVE} - slave reception mode</li>
   * <li>{@link #LIN_OPMODE_MASTER} - master transmission and slave reception
   * mode</li>
   * </blockquote> Optionally, the following flags can be combined with the
   * primary operating mode flags: <blockquote>
   * <li>{@link #LIN_OPMODE_ERRORS} - enable reception of error frames</li>
   * </blockquote> The bit transfer rate for the LIN controller is specified by
   * the wLinBitrate parameter. The timing can be detected automatically by
   * using {@link #LIN_BITRATE_AUTO}. If an automatic detection is not possible
   * or wished a predefined Bit rate from the list between
   * {@link #LIN_BITRATE_MIN} and {@link #LIN_BITRATE_MAX} can be used .
   * </blockquote>
   * 
   * @param bOperatingMode
   *          Operating mode of the LIN line. Refer to {@link #LIN_OPMODE_ERRORS}
   * @param wLinBitrate
   *          Bit rate of the LIN controller. Refer to {@link #LIN_BITRATE_MIN}
   * @throws Throwable
   */
  public void InitLine(byte bOperatingMode, short wLinBitrate) throws Throwable;

  /**
   * This method reset the LIN line to it's initial state. The method aborts a
   * currently busy transmit message and switch the LIN controller into init
   * mode. 
   * 
   * @throws Throwable
   */
  public void ResetLine() throws Throwable;

  /**
   * This method starts the LIN line and switch it into running mode. After
   * starting the LIN line, LIN messages can be received over the message
   * channel.
   * 
   * @throws Throwable
   */
  public void StartLine() throws Throwable;

  /**
   * This method stops the LIN line an switch it into init mode. After stopping
   * the LIN controller no further LIN messages are transmitted. Other than
   * ResetLine, this method does not abort a currently busy transmit message.
   * 
   * @throws Throwable
   */
  public void StopLine() throws Throwable;

  /**
   * This method puts the LIN message in the response table or transmits the
   * message on the LIN bus. If fSend is set to <code>FALSE</code> the message
   * is put into the response table. By setting fSend to <code>TRUE</code> the
   * message is sent directly on the LIN bus.
   * 
   * @param fSend
   *          Flag specifying whether to put the message in the response table
   *          oder send the message directly on the bus.
   * @param oLinMsg
   *          LIN Message to put in response table or to send directly on the
   *          LIN bus.
   * @throws Throwable
   */
  public void WriteMessage(boolean fSend, LinMessage oLinMsg) throws Throwable;

  /**
   * Gets the current status of the LIN Line.
   * 
   * @return {@link LinLineStatus} LinLineStatus is used to signal the status of
   *         a LIN controller.
   * @throws Throwable
   */
  public LinLineStatus GetLineStatus() throws Throwable;
}
