/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the interface {@link ILinMonitor} which represents a
 * LIN monitor channel and is used to receive LIN messages.
 * 
 * @see ILinMonitor
 * @author Michael Ummenhofer
 * 
 */
public class LinMonitor extends CInterface implements ILinMonitor
{
  private static final int Initialize = 3;
  private static final int Activate   = 4;
  private static final int Deactivate = 5;
  private static final int GetReader  = 6;
  private static final int GetStatus  = 7;

  // Interface functions
  private CFunction m_fpInitialize  = null;
  private CFunction m_fpActivate    = null;
  private CFunction m_fpDeactivate  = null;
  private CFunction m_fpGetReader   = null;
  private CFunction m_fpGetStatus   = null;

  // Interface Pointer
  private CPointer m_ILinMonitor = null; // Interface Pointer itself (pThis)
  
  /** LIN Channel Status */
  private LinMonitorStatus m_oLinMonitorStatus = new LinMonitorStatus();

  /**
   * constructor
   * 
   * @param pILinMonitor
   * @throws Throwable
   */
  protected LinMonitor(CMalloc pILinMonitor) throws Throwable
  {
    super(pILinMonitor);

    try
    {
      // Retrieve Interface Pointer Address
      m_ILinMonitor = super.m_Interface;

      // Retrieve Interface Methods
      m_fpInitialize  = new CFunction(m_ILinMonitor, Initialize,  "JNI");
      m_fpActivate    = new CFunction(m_ILinMonitor, Activate,    "JNI");
      m_fpDeactivate  = new CFunction(m_ILinMonitor, Deactivate,  "JNI");
      m_fpGetReader   = new CFunction(m_ILinMonitor, GetReader,   "JNI");
      m_fpGetStatus   = new CFunction(m_ILinMonitor, GetStatus,   "JNI");
    } 
	catch (Exception oException)
    {
      // Release all Pointers without freeing memory
      m_ILinMonitor   = null;
      m_fpInitialize  = null;
      m_fpActivate    = null;
      m_fpDeactivate  = null;
      m_fpGetReader   = null;
      m_fpGetStatus   = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   * 
   * @throws Throwable
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    } 
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ILinMonitor   = null;
      m_fpInitialize  = null;
      m_fpActivate    = null;
      m_fpDeactivate  = null;
      m_fpGetReader   = null;
      m_fpGetStatus   = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }

  /**
   * @see ILinMonitor#Initialize(short)
   * @throws Throwable
   */
  public void Initialize(short wRxSize) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpInitialize.callInt(new Object[] { m_ILinMonitor, new Integer(wRxSize) });
      if (hResult != 0)
      {
        throw new VciException("ILinMonitor->Initialize failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ILinMonitor#Activate()
   * @throws Throwable
   */
  public void Activate() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpActivate.callInt(new Object[] { m_ILinMonitor });
      if (hResult != 0)
      {
        throw new VciException("ILinMonitor->Activate failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ILinMonitor#Deactivate()
   * @throws Throwable
   */
  public void Deactivate() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpDeactivate.callInt(new Object[] { m_ILinMonitor });
      if (hResult != 0)
      {
        throw new VciException("ILinMonitor->Deactivate failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ILinMonitor#GetMessageReader()
   * @return {@link ILinMessageReader}
   * @throws Throwable
   */
  public ILinMessageReader GetMessageReader() throws Throwable
  {
    LinMessageReader oLinMessageReader = null;
    int hResult = 0;

    // Create LinMessageReader Interface Memory
    CMalloc pILinMessageReader = new CMalloc(8);
    try
    {
      pILinMessageReader.setLong(0, 0);

      // Retrieve IFifoReader Interface
      hResult = m_fpGetReader.callInt(new Object[] { m_ILinMonitor, pILinMessageReader });
      if (hResult != 0)
      {
        throw new VciException("ILinMonitor->GetReader failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
	  else
      {
        // Create new LinMessageReader Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the LinMessageReader Object
        oLinMessageReader = new LinMessageReader(pILinMessageReader);
      }
    } 
	catch (Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pILinMessageReader.free();
      throw oException;
    } 
	finally
    {
    }

    return oLinMessageReader;
  }

  /**
   * @see ILinMonitor#GetStatus()
   * @throws Throwable
   */
  public LinMonitorStatus GetStatus() throws Throwable
  {
    int hResult = 0;

    // Create LINMONITORSTATUS Memory
    CMalloc pLinMonitorStatus = new CMalloc(17);
    try
    {
      hResult = m_fpGetStatus.callInt(new Object[] { m_ILinMonitor, pLinMonitorStatus });
      if (hResult != 0)
      {
        throw new VciException("ILinMonitor->GetStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oLinMonitorStatus.fromCPointer(pLinMonitorStatus);
      }
    } 
    finally
    {
      pLinMonitorStatus.free();
    }
    
    return m_oLinMonitorStatus;
  }

}
